package no.ubicollab.osgi.service.collaboration;

/**
 * An interface for the Collaboration Service. Provides collaboration instance access to 
 * applications and other devices through web service calls.
 * 
 * @author 	Christian H. Mosveen
 * @version	0.9
 * @since	0.9
 */
public interface CollaborationService {
	
	/**
	 * Creates a collaboration instance with the specified collabInstName, and where the creator
	 * is specified by username. 
	 * 
	 * @param	username		The username of the collaboration instance's creator
	 * @param	collabInstName	The name of the collaboration instance to create
	 * @return					A unique hash representing the collab inst id
	 */
	public String createCollaborationInstance(String username, String collabInstName);
	
	/**
	 * Removes a specified collaboration instance, if the username specified matches the
	 * collaboration instance's owner.
	 * 
	 * @param	collabInstId	The id of the collaboration instance to remove
	 * @param 	username		The username of the person calling the method
	 * @return					True if the specified collab inst was removed, false otherwise
	 */
	public boolean removeCollaborationInstance(String collabInstId, String username);
	
	/**
	 * Adds a specified person to the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance to add to
	 * @param 	username		The username of the person to add
	 * @return					True if the person was added to the collab inst, false otherwise
	 */
	public boolean addPersonToCollabInst(String collabInstId, String username);
	
	/**
	 * Removes a specified person from the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance to remove from
	 * @param	username		The username of the person to remove
	 * @return					True if the person was removed from the collab inst, false otherwise
	 */
	public boolean removePersonFromCollabInst(String collabInstId, String username);
	
	/**
	 * Adds a specified service to the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance to add to
	 * @param 	descriptionUrl	The descriptionUrl of the service to add
	 * @return					True if the service was added to the collab inst, false otherwise
	 */
	public boolean addServiceToCollabInst(String collabInstId, String descriptionUrl);
	
	/**
	 * Removes a specified service from the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance to remove from
	 * @param	descriptionUrl	The descriptionUrl of the service to remove
	 * @return					True if the service was removed from the collab inst, false otherwise
	 */
	public boolean removeServiceFromCollabInst(String collabInstId, String descriptionUrl);
	
	/**
	 * @param collabInstId
	 * @param descriptionUrl
	 * @param username
	 * @param appName
	 * @param fileName
	 * @return
	 */
	public boolean addFileToCollabInst(String collabInstId, String username, String appName, String fileName);
	
	/**
	 * @param collabInstId
	 * @param descriptionUrl
	 * @param username
	 * @param appName
	 * @param fileName
	 * @return
	 */
	public boolean removeFileFromCollabInst(String collabInstId, String username, String appName, String fileName);
	
	/**
	 * Gets a list of all registered collaboration instances
	 * 
	 * @return	A String[] with the id's of all registered collaboration instances
	 */
	public String[] getAllCollaborationInstances();
	
	/**
	 * Gets a list of all registered persons
	 * 
	 * @return	A String[] with the usernames of all registered persons	
	 */
	public String[] getAllPersons();
	
	/**
	 * Gets a list of all registered services
	 * 
	 * @return	A String[] with the descriptionUrl's of all registered services
	 */
	public String[] getAllServices();
	
	/**
	 * Gets a list of all the files of a person
	 * @param 	username	The username of the person whose file list should be returned
	 * @return				A String[] with the fileId's of all of the person's files.
	 * 						The fileId is represented as: 
	 * 						&lt;username&gt;/&lt;appName&gt;/&lt;fileName&gt;
	 */
	public String[] getPersonsFiles(String username);
	
	/**
	 * Gets a list of all registered collaboration instances a person is member of
	 * @param 	username	The username of the person whose collaboration instances 
	 * 						should be returned
	 * @return				A String[] with the id's of all the collaboration instances
	 * 						the person is a member of
	 */
	public String[] getPersonsCollaborationInstances(String username);
	
	/**
	 * Gets a list of all persons in the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance
	 * @return					A String[] with the usernames of all persons 
	 * 							in the collaboration instance
	 */
	public String[] getAllPersonsInCollabInst(String collabInstId);
	
	/**
	 * Gets a list of all services in the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance
	 * @return					A String[] with the descriptionUrl's of all services 
	 * 							in the collaboration instance
	 */
	public String[] getAllServicesInCollabInst(String collabInstId);
	
	/**
	 * Gets a list of all files in the specified collaboration instance
	 * @param 	collabInstId	The id of the collaboration instance
	 * @return					A String[] with the fileId's of all files 
	 * 							in the collaboration instance
	 */
	public String[] getAllFilesInCollabInst(String collabInstId);
	
	/**
	 * Gets the name of the specified collaboration instance
	 * 
	 * @param 	collabInstId	The id of the collaboration instance
	 * @return					The name of the collaboration instance
	 */
	public String getCollabInstName(String collabInstId);
	
	/**
	 * Gets the full name of the specified person
	 * 
	 * @param 	username	The person's username
	 * @return				The person's full name
	 */
	public String getPersonsFullName(String username);
	
	/**
	 * Gets all registered information about the specified service
	 * 
	 * @param 	url		The descriptionUrl or serviceUrl of the service
	 * @return			A String[] with a length of 10, with the fields: 
	 * 					dbId, tagId, uuId, name, type, protocol, 
	 * 					descriptionUrl, serviceUrl, owner, discovered.
	 */
	public String[] getInfoAboutService(String url);
}
