package no.ubicollab.osgi;

import no.ubicollab.osgi.service.discovery.DiscoveryService;

import java.sql.*;

import org.w3c.dom.*;

import javax.xml.parsers.*;

import org.xml.sax.*;

import java.io.*;

public class DiscoveryImpl implements DiscoveryService {

	private Connection sqlConn = null;

	private boolean serviceStopped = false;

	public DiscoveryImpl() {
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			sqlConn = connectToDatabase();
			System.out.println("Connected to DB");
			ServiceRemovalThread thread = new ServiceRemovalThread(this,
					sqlConn);
			thread.start();
			System.out.println("Service removal process is started");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 * Called when the bundle is stopped. Closes the database connection, and
	 * disallows further contact with it.
	 */
	public void disconnectFromDatabase() {
		try {
			sqlConn.close();
			serviceStopped = true;
			System.out.println("Disconnected from DB");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 * Searches for services in the database, and returns any matches. The
	 * format of the search string is as follows: tagId:<tagId> uuId:<uuId>
	 * name:<name> type:<type> protocol:<protocol> descriptionUrl:<descriptionUrl>
	 * serviceUrl:<serviceUrl> Any number of these flags may be present, but
	 * they must come in the right order.
	 * 
	 * The method returns either "Service is stopped", "No matching services",
	 * or a list of service/protocol pairs represented like this: "service1
	 * protocol1 service2 protocol2"
	 * 
	 * @see no.ubicollab.osgi.service.DiscoveryService#getService(java.lang.String)
	 */
	public String getService(String searchString) {
		if (serviceStopped) {
			return "Service is stopped";
		}
		String protocolVal = "";
		String descriptionVal = "";
		String serviceVal = "";
		int count = 0;
		String serviceList = "";

		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = DiscoveryUtils.buildGetServiceQuery(searchString);
			if (query.equals("")) {
				return "No matching services";
			}
			s.executeQuery(query);
			ResultSet rs = s.getResultSet();
			while (rs.next()) {
				protocolVal = rs.getString("protocol");
				descriptionVal = rs.getString("descriptionUrl");
				serviceVal = rs.getString("serviceUrl");

				if (protocolVal.equals("ws")) {
					serviceList += serviceVal + " ws ";
					count++;
				} else if (protocolVal.equals("upnp")) {
					serviceList += descriptionVal + " upnp ";
					count++;
				}

			}
			rs.close();
			s.close();
			serviceList = serviceList.trim();
			// System.out.println (count + " rows were retrieved");
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}

		if (count == 0) {
			return "No matching services";
		} else if (count > 0) {
			return serviceList;
		} else {
			return "Error";
		}

	}

	/*
	 * Returns information about a specified service as a String array. The
	 * array has a length of 10, with the fields: dbId, tagId, uuId, name, type,
	 * protocol, descriptionUrl, serviceUrl, owner, discovered.
	 * 
	 * The service can be specified both by the service URL (web services), and
	 * by the description URL (UPnP).
	 * 
	 * @see no.ubicollab.osgi.service.discovery.DiscoveryService#getInfoAboutService(java.lang.String)
	 */
	public String[] getInfoAboutService(String url) {
		String[] serviceInfo = null;
		Statement s = null;
		ResultSet rs = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			String query = "SELECT * FROM discovery WHERE serviceUrl = '" + url
					+ "'";
			s.executeQuery(query);
			rs = s.getResultSet();
			if (rs.next()) {
				serviceInfo = new String[10];
				serviceInfo[0] = "" + rs.getInt("id");
				serviceInfo[1] = rs.getString("tagId");
				serviceInfo[2] = rs.getString("uuId");
				serviceInfo[3] = rs.getString("name");
				serviceInfo[4] = rs.getString("type");
				serviceInfo[5] = rs.getString("protocol");
				serviceInfo[6] = rs.getString("descriptionUrl");
				serviceInfo[7] = rs.getString("serviceUrl");
				serviceInfo[8] = rs.getString("owner");
				serviceInfo[9] = rs.getTimestamp("discovered").toString();
			} else {
				query = "SELECT * FROM discovery WHERE descriptionUrl = '"
						+ url + "'";
				s.executeQuery(query);
				rs = s.getResultSet();
				if (rs.next()) {
					serviceInfo = new String[10];
					serviceInfo[0] = "" + rs.getInt("id");
					serviceInfo[1] = rs.getString("tagId");
					serviceInfo[2] = rs.getString("uuId");
					serviceInfo[3] = rs.getString("name");
					serviceInfo[4] = rs.getString("type");
					serviceInfo[5] = rs.getString("protocol");
					serviceInfo[6] = rs.getString("descriptionUrl");
					serviceInfo[7] = rs.getString("serviceUrl");
					serviceInfo[8] = rs.getString("owner");
					serviceInfo[9] = rs.getTimestamp("discovered").toString();
				}
			}
			rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}

		return serviceInfo;
	}

	/*
	 * Registers a service described by the specified URL
	 * 
	 * @see no.ubicollab.osgi.service.DiscoveryService#registerService(java.lang.String)
	 */
	public boolean registerService(String url) {
		if (serviceStopped) {
			return false;
		}
		boolean response = parseAndInsertService("", url);
		return response;
	}

	/*
	 * Registers a service described by the specified URL, and discovered with
	 * the specified tag id.
	 * 
	 * @see no.ubicollab.osgi.service.DiscoveryService#registerService(java.lang.String,
	 *      java.lang.String)
	 */
	public boolean registerService(String tagId, String url) {
		if (serviceStopped) {
			return false;
		}
		boolean response = parseAndInsertService(tagId, url);
		return response;
	}

	/*
	 * Registers a service with all parameters set. This can be useful when the
	 * service description can't be parsed.
	 * 
	 * @see no.ubicollab.osgi.service.DiscoveryService#registerService(java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean registerService(String tagId, String uuId, String name,
			String type, String protocol, String descriptionUrl,
			String serviceUrl, String owner) {
		if (serviceStopped) {
			return false;
		}

		boolean serviceExists = checkExistingServices(descriptionUrl);
		if (serviceExists) {
			System.out.println("Service already existed: " + descriptionUrl);
			return false;
		}

		int count = executeRegistration(tagId, uuId, name, type, protocol,
				descriptionUrl, serviceUrl, owner);

		if (count == 1) {
			System.out.println("Registered " + descriptionUrl);
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Removes all services matching the remove string from the database
	 * 
	 * @see no.ubicollab.osgi.service.DiscoveryService#removeService(java.lang.String)
	 */
	public boolean removeService(String removeString) {
		if (serviceStopped) {
			return false;
		}
		String query = DiscoveryUtils.buildRemoveServiceQuery(removeString);

		int count = 0;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			count = s.executeUpdate(query);
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}

		if (count == 0) {
			return false;
		} else {
			System.out.println("Removing " + removeString);
			return true;
		}
	}

	/*
	 * Parses the document specified by the URL, and registers the service if
	 * that document matches any known service protocol. Can currently parse
	 * WSDL-documents (web services), and UPnP description XML's.
	 */
	private boolean parseAndInsertService(String tagId, String url) {

		boolean serviceExists = checkExistingServices(url);
		if (serviceExists) {
			System.out.println("Service already existed: " + url);
			return false;
		}

		String uuId = "";
		String serviceName = "";
		String serviceType = "";
		String serviceProtocol = "";
		String serviceUrl = "";
		String owner = "";
		int count = 0;

		Document descriptionXml;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			descriptionXml = builder.parse(url);
			Node rootNode = descriptionXml.getFirstChild();
			String rootName = rootNode.getNodeName();
			if (rootName.startsWith("wsdl")) {
				serviceProtocol = "ws";
				NodeList serviceNodes = descriptionXml
						.getElementsByTagName("wsdl:service");
				Node serviceNode = serviceNodes.item(0);
				Node portNode = serviceNode.getFirstChild().getNextSibling();
				serviceName = ((Element) portNode).getAttribute("name");
				Node addressNode = portNode.getFirstChild().getNextSibling();
				serviceUrl = ((Element) addressNode).getAttribute("location");
			} else if (rootName.startsWith("root")) {
				String ns = ((Element) rootNode).getAttribute("xmlns");
				if (ns != null && ns.startsWith("urn:schemas-upnp-org")) {
					serviceProtocol = "upnp";
					NodeList deviceNodes = descriptionXml
							.getElementsByTagName("device");
					Node deviceNode = deviceNodes.item(0);
					Node typeNode = deviceNode.getFirstChild().getNextSibling();
					serviceType = typeNode.getFirstChild().getNodeValue();
					Node nameNode = typeNode.getNextSibling().getNextSibling();
					serviceName = nameNode.getFirstChild().getNodeValue();
					NodeList udnNodes = descriptionXml
							.getElementsByTagName("UDN");
					Node udnNode = udnNodes.item(0);
					uuId = udnNode.getFirstChild().getNodeValue();

				}
			}
			count = executeRegistration(tagId, uuId, serviceName, serviceType,
					serviceProtocol, url, serviceUrl, owner);

		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		} catch (SAXException saxe) {
			saxe.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		if (count == 1) {
			System.out.println("Registered " + url);
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Registers a service with the specified fields. Returns the number of
	 * updates to the database. 1 if the service was registered successfully, 0
	 * otherwise.
	 */
	private int executeRegistration(String tagId, String uuId, String name,
			String type, String protocol, String descriptionUrl,
			String serviceUrl, String owner) {
		int count = 0;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			count = s
					.executeUpdate("INSERT INTO discovery(tagId, uuId, name, type, protocol, descriptionUrl, serviceUrl, owner)"
							+ " VALUES"
							+ "('"
							+ tagId
							+ "',"
							+ "'"
							+ uuId
							+ "',"
							+ "'"
							+ name
							+ "',"
							+ "'"
							+ type
							+ "',"
							+ "'"
							+ protocol
							+ "',"
							+ "'"
							+ descriptionUrl
							+ "',"
							+ "'"
							+ serviceUrl
							+ "',"
							+ "'"
							+ owner
							+ "')");
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		return count;
	}

	/*
	 * Checks if the specified service already exists. Returns true if it does,
	 * false if it does not.
	 */
	private boolean checkExistingServices(String url) {
		boolean serviceExists = false;
		Statement s = null;
		ResultSet rs = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			String query = "SELECT * FROM discovery WHERE descriptionUrl = '"
					+ url + "'";
			s.executeQuery(query);
			rs = s.getResultSet();
			if (rs.next()) {
				serviceExists = true;
			} else {
				serviceExists = false;
			}
			rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		return serviceExists;
	}

	/*
	 * Connects to the database. Returns the connection handle.
	 */
	public Connection connectToDatabase() {
		try {
			sqlConn = DriverManager.getConnection(
					"jdbc:mysql://localhost/ubicollab", "ubi", "");
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		return sqlConn;
	}

}