package no.ubicollab.osgi;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Enumeration;

import javax.comm.CommPortIdentifier;
import javax.comm.SerialPort;

//driver used on pda can be found here: http://www2s.biglobe.ne.jp/~dat/java/project/jvm/index_en.html (only comm api not entire JVM)
//Note that the name of the comport seems to end with ":" on the pda eg. COM7:
//This is not the case for win32 eg. COM7
/**
 * Class for accessing and retrieving positiondata from a gps device via the com-port.
 * It requires the java comm api to be installed on the platform where it is run.<br>
 * An implementation of the java comm api for Pocketpc can be found here: <a href="http://www2s.biglobe.ne.jp/~dat/java/project/jvm/index_en.html">http://www2s.biglobe.ne.jp/~dat/java/project/jvm/index_en.html</a> (Download the comm api only. The whole JVM is not needed).<br><br>
 * 
 * This class has been tested with the ITerNet PS-3100 Bluetooth GPS, on Windows Xp and Windows Mobile 2003, 
 * but should also work with other GPS devices as long as they can be accessed through a com-port and the GPS returns $GPRMC commands from the NMEA-0183 GPS standard (see Parser for details)  
 * @author Andreas Brustad
 * @see Parser
 */
public class SerialPortGPS {
	
	private String comPort;
	private SerialPort sp;
	private InputStreamReader reader;
	private int TIMEOUT=50;
	//private OutputStream out;

	/**
	 * Constructs a new SerialPortGPS object with connection to the specified comport 
	 * @param comPort The comport to access the GPS on. Note that name of the comport seems to end with ":" on the pda eg. COM7: 
	 * This is not the case for win32 eg. COM7
	 */
	/**
	 * @param comPort
	 */
	public SerialPortGPS(String comPort)
	{
		this.comPort=comPort;
	}
	/**
	 * Gets the position from a GPS device.<br>
	 * Opens a connection to the GPS, retrieves the position from it and closes the connection.
	 * @return a String[] of length 3. Where [0] = longitude, [1] = latitude, [2] = altitude, or null if no contact with the GPS could be estblished.
	 */
	public LocationData getPosition(){
		
		try
		{
			System.out.println("<GpsPositioning> accessing GPS on "+comPort);
			CommPortIdentifier portId = null;
		    for (Enumeration e = CommPortIdentifier.getPortIdentifiers() ; e.hasMoreElements() ;) {
		      CommPortIdentifier tmp = (CommPortIdentifier)e.nextElement();
		      //System.out.println("Found port"+tmp.getName());
		      if (tmp.getName().equals(comPort)) {
		        portId = tmp;
		        break;
		      }
		    }
		    if (portId == null) {
		      System.out.println("Unable to find" +comPort);
		      return null;
		    }
		    
		    //ComPort found !
			sp = (SerialPort)portId.open("GPSPlugin", 0);

			sp.setSerialPortParams(9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			sp.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);

			reader = new InputStreamReader(sp.getInputStream());
			//out = sp.getOutputStream();
			
		}
		catch (Exception e) 
		{
			System.out.println("Could not connect. Exception " + e);
			e.printStackTrace();
			close(reader);
			return null;
		}
		
		
		int input;
		int timeoutValue=0;
		
		try
		{
			while(timeoutValue<this.TIMEOUT)
			{
				String output=new String();
				input=0;
				//13 = \r
				while ((input = reader.read()) != 13)
				{
					output += (char) input;
					
				}
				//get rid of \n
				reader.read();
				
				timeoutValue++;
				//System.out.println(output);
				LocationData locDat=Parser.parse(output);
				if(locDat!=null)
				{
					close(reader);
					//return 0m altitude in this version
					return locDat;
				}
			}
			System.out.println("Timed out");
			close(reader);
			return null;
		}
		catch(IOException ioe)
		{
			System.out.println("Fikk ikke kobla til");
			close(reader);
			return null;
		}
	}
	
	/**
	 * Sets the maximum number unsuccessfully parsed commands to read from the GPS before timeout. Default is 50   
	 * @param timeoutValue Max attempts before timeout
	 */
	public void setTimeout(int timeoutValue){
		this.TIMEOUT=timeoutValue;
	}
	
	private void close(InputStreamReader reader){
		try{		
			reader.close();
			sp.close();
		}
		catch(Exception e)
		{
			System.out.println("Fikk ikke lukka bluetoothconnection");
		}
	}
}
