package no.ubicollab.osgi.service.pocketdiscovery;

/**
 * An interface for the Pocket Discovery Service. 
 * Pocket Discovery Service is a lightweight version of the Discovery Service designed to run on a pocket pc or similar device.<br>
 * Except from the registerDiscoveryService(String url) -method, the service does no job on its own, and instead forwards all requests to a Discovery Service. 
 * The url of the Discovery Service to forward to is read from a config file at startup, but can also be set at runtime via the registerDiscoveryService(String url) -method.
 * 
 * @author Andreas Brustad
 * @see no.ubicollab.osgi.service.discovery.DiscoveryService
 *
 */
public interface PocketDiscoveryService {
	
	//Call this method to update the url to the discoveryService
	/**
	 * Sets the url of the Discovery Service to forward requests to. The url is written to file so that the url is kept in case the service has to restart.
	 * @param url Url of the Discovery Service to forward requests to.
	 * @return true if url was successfully updated and written to file, false otherwise
	 */
	public boolean registerDiscoveryService(String url);
	
	/**
	 * Registers a service. Use this method if you have an RFID tag with the URL to a 
	 * WSDL-file or to the description-XML of a UPnP service.
	 * 
	 * @param 	tagId	The id of the RFID tag scanned to get the service.
	 * @param 	url		The descriptionUrl of the service.
	 * @return			True if the service is successfully registered, false otherwise.
	 */
	public boolean registerService(String tagId, String url);
	
	/**
	 * Registers a service. Use this method if you have the URL to a WSDL-file or to the 
	 * description XML of a UPnP service.
	 * 
	 * @param 	url		The descriptionUrl of the service.
	 * @return			True if the service is successfully registered, false otherwise.
	 */
	public boolean registerService(String url);
	
	/**
	 * Registers a service. Use this method if you have more information about the service
	 * that you want to add with it.
	 * 
	 * @param 	tagId			The id of the RFID tag scanned to get the service.
	 * @param 	uuId			The id of the UPnP device providing the service.
	 * @param 	name			The name of the service.
	 * @param 	type			The service type.
	 * @param 	protocol		The service protocol.
	 * @param 	descriptionUrl	The URL describing the service (ie. the URL to a WSDL-file).
	 * @param 	serviceUrl		The service URL of the service (ie. http://abc.com/axis/services/ubiService)
	 * @param 	owner			The person owning the service.
	 * @return					True if the service is successfully registered, false otherwise.
	 */
	public boolean registerService(String tagId, String uuId, String name, String type, String protocol, String descriptionUrl, String serviceUrl, String owner);
	
	/**
	 * Searches for services in the database, and returns any matches. 
	 *
	 * @param 	searchString	tagId:&lt;tagId&gt; uuId:&lt;uuId&gt; name:&lt;name&gt; type:&lt;type&gt; protocol:&lt;protocol&gt;
	 * 							descriptionUrl:&lt;descriptionUrl&gt; serviceUrl:&lt;serviceUrl&gt;
	 * 							Any number of these flags may be present, but they must come 
	 * 							in the right order. Example: "name:My Service type:no.ubicollab.service"
	 * 
	 * @return					The method returns either "Service is stopped", "No matching services", 
	 * 							or a list of service/protocol pairs represented like this: 
	 * 							"service1 protocol1 service2 protocol2"
	 */
	public String getService(String searchString);
	
	/**
	 * Removes all services matching the removeString.
	 * 
	 * @param 	removeString	tagId:&lt;tagId&gt; uuId:&lt;uuId&gt; name:&lt;name&gt; type:&lt;type&gt; protocol:&lt;protocol&gt;
	 * 							descriptionUrl:&lt;descriptionUrl&gt; serviceUrl:&lt;serviceUrl&gt;
	 * 							Any number of these flags may be present, but they must come 
	 * 							in the right order. Example: "name:My Service type:no.ubicollab.service"
	 * @return					True if any services are successfully removed, false otherwise.
	 */
	public boolean removeService(String removeString);
	
	/**
	 * Gets all registered information about the specified service.
	 * 
	 * @param 	url		Either the descriptionUrl or the serviceUrl of the service.
	 * @return			A String[] with a length of 10, with the fields: 
	 * 					dbId, tagId, uuId, name, type, protocol, 
	 * 					descriptionUrl, serviceUrl, owner, discovered.
	 */
	public String[] getInfoAboutService(String url);
	
}
