package no.ubicollab.osgi;

import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Definition;
import javax.wsdl.Part;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;


/**
 * Class for generating webservice proxy classes to plugins at runtime
 * @author Andreas Brustad
 *
 */
public class DynamicProxyGenerator{
	
	WSDLReader wsdlReader;
	
	public DynamicProxyGenerator(){
		try {
			WSDLFactory wsdlFactory = WSDLFactory.newInstance();
			wsdlReader = wsdlFactory.newWSDLReader();
			wsdlReader.setFeature("javax.wsdl.verbose",false);
		} catch (WSDLException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Generates proxyclasses to webservices that implement the PositioningPluginService interface. 
	 * The method checks for the double[] getPosition(String) -method in the wsdl description. If found, a proxy 
	 * is generated to this webservice.
	 * @param wsdlUrl The url to the wsdl description of the plugin webservice
	 * @return A proxy that can be used to invoke the webservice methods of the plugin, or null if the 
	 * webservice does not implement the PositioningPluginService interface.
	 */
	public PositioningPluginService getProxy(String wsdlUrl){
		//System.out.println("");
		System.out.println("<PositioningService>Parsing wsdl file..");	
		Definition definition;
			try {
				definition = wsdlReader.readWSDL(wsdlUrl);
			} catch (WSDLException e) {
				System.out.println("<PositioningService>Error parsing wsdl file with url: "+wsdlUrl);
				//e.printStackTrace();
				return null;
			}
			Map servicesMap = definition.getServices();
			Set s = servicesMap.keySet();
			Iterator it = s.iterator();
			
			//check all combinations of services/ports/operations to see if we have a compatible service
			
			//for all services
			while(it.hasNext()) {
				javax.wsdl.Service serv =(javax.wsdl.Service) servicesMap.get(it.next());
				QName serviceQname= serv.getQName();
				Map m= serv.getPorts();
				Set set= m.keySet();
				Iterator itr = set.iterator();
				
				//for all ports
				while(itr.hasNext()) {
					List ops;
					QName portQname;
					try{
						javax.wsdl.Port port = (javax.wsdl.Port) m.get(itr.next());
						
						 portQname= new QName(serviceQname.getNamespaceURI(),port.getName());
						//System.out.println("Found service: "+serviceQname.getLocalPart()+" with port:"+portQname.getLocalPart());
						//System.out.println("Testing to see if it is compatible");
						
						//check if method name is correct, that is see if it is "getPosition"
						 ops= port.getBinding().getPortType().getOperations();
					}
					catch(Exception e){
						System.out.println("<PositioningService>Incompatible service/port. Exception while parsing");
						e.printStackTrace();
						continue;
					}
					//for all operations (methods)
					for(int i =0; i<ops.size(); i++){
						try{	
							javax.wsdl.Operation op = (javax.wsdl.Operation)ops.get(i);
							//check if methodname = getPosition
							if(!"getPosition".equals(op.getName())){
								//System.out.println("Incompatible service/port/operation. Not a getPosition method");
								continue;
							}
							//check if method takes one inputparameter
							Collection params =op.getInput().getMessage().getParts().values();
							if(params.size()!=1){
								//System.out.println("Incompatible service/port/operation. Does not take one input parameter");
								continue;
							}
							//check if inputparameter is of type string
							Iterator iter = params.iterator();
							Part param = (Part)iter.next();
							if(!"string".equals(param.getTypeName().getLocalPart())){
								//System.out.println("Incompatible service/port/operation. Input parameter not of type string");
								continue;
							}
							//check if method returnes one parameter
							Collection returntypes =op.getOutput().getMessage().getParts().values();
							if(returntypes.size()!=1){
								//System.out.println("Incompatible service/port/operation. Does not return one parameter");
								continue;
							}
							//check if returntype is double[]
							Iterator outiter = returntypes.iterator();
							Part outparam = (Part)outiter.next();
							if(!"ArrayOf_xsd_double".equals(outparam.getTypeName().getLocalPart())){
				            	//System.out.println("Incompatible service/port. Does not return double[]");
								continue;
							}
														
							//Service ok. Create proxy
							
							/* Service lookup */
				            ServiceFactory serviceFactory = ServiceFactory.newInstance();
				            Service sampleService = serviceFactory.createService(
				                new URL(wsdlUrl), serviceQname);
				            				            
				            /* Service access */
				            //System.out.println("Classloader in DynPG "+this.getClass().getClassLoader());
				            //System.out.println("Service is"+sampleService);
				            Class test = PositioningPluginService.class;
				            PositioningPluginService myProxy = (PositioningPluginService) sampleService.getPort(
				                portQname, test,this.getClass().getClassLoader());
				            
				            
				            System.out.println("<PositioningService>Found compatible service. Returning proxy");
				            return myProxy;
						}
						catch(Exception e){
							System.out.println("<PositioningService>Incompatible service/port. Exception while generating proxy");
							e.printStackTrace();
						}
					}
				}
			}
			System.out.println("<PositioningService>No compatible service/ports found");
			return null;
	}
} 