/**
 * Title: Implementation for a UPnP Binary Light Device<p>
 * (c) 2001, Siemens AG<p>
 *
 * @author Markus Alexander Wischy
 * @version 1.0
 */

package com.siemens.upnp.testdevice;

import com.siemens.upnp.*;
import com.siemens.upnp.devicehost.*;
import com.siemens.upnp.util.*;
import com.siemens.upnp.testdevice.ui.*;
import com.siemens.upnp.testdevice.service.*;
import java.io.*;

public class BinaryLightDevice extends HostedDevice implements SwitchPower {

    static BinaryLightFrame frame;
    SwitchPowerImpl switchPowerImpl;

    public BinaryLightDevice(String uuid, String friendlyName) {
        // create dynamically the test device
        this.createDevice(uuid, friendlyName);

        this.initDevice();
    }

    public BinaryLightDevice(File file)
            throws InvalidDescriptionException, FileNotFoundException {
        super(file);

        this.initDevice();
    }

    public void shutDown() {
        // shut down the DeviceHost (WebServer, SSDP, GENA)
        DeviceHost.getInstance().shutDown();
    }

    private void initDevice() {
        // create the user interface
        frame = new BinaryLightFrame(this);

        // required for root devices
        this.setRootDevice(true);
        this.setLeaseTime(3600);

        // insantiate power switch implementation
        switchPowerImpl = new SwitchPowerImpl(this);
        switchPowerImpl.addListener(this);

        // create a handler for the presentation page
        UPnPPresentationPageHandler handler =
            new BinaryLightPresentationPageHandler(this, switchPowerImpl);

        this.setPresentationPageHandler(handler);
    }

    // update from user interface
    public void updateStatus(boolean newStatus) {
        switchPowerImpl.setTarget(newStatus);
    }

    // implementation of swithc power interface
    public void setTarget(boolean newStatus) {
        frame.updateUI(newStatus);
    }

    // implementation of swithc power interface (not called)
    public boolean getStatus() {
        return false;
    }

    /**
     * Creates dynamically a device
     *
     * @param udn The device UDN
     * @param friendlyName The friendly name of the device
     */
    public void createDevice(String udn, String friendlyName) {

        // required attributes
        if (udn != null) this.setUDN(udn);
        else this.setUDN("uuid:siemensTestDevice");

        if (friendlyName != null) this.setFriendlyName(friendlyName);
        else this.setFriendlyName("Siemens Test Device");

        this.setDeviceType("urn:schemas-upnp-org:device:binarylight:1");
        this.setManufacturer("Siemens CT SE 2");
        this.setModelName("Vanilla");

        // recommend attributes
        this.setModelDescription("A Test Device for the UPnP Stack Implementation");
        this.setModelNumber("1.0");
        this.setSerialNumber("1234567890");

        // optional attributes
        this.setManufacturerURL("/manufacturer.html");
        this.setModelURL("/model.html");
        this.setUPC("123456789012");

        // icons
        Icon icon = new Icon("icon.gif");
        icon.setMimeType("image/gif");
        icon.setHeight(30);
        icon.setWidth(30);
        icon.setColorDepth(8);

        this.addIcon(icon);

        // embedded devices
        // go here

        // services
        HostedService service = this.createService();
        service.setServiceType("urn:schemas-upnp-org:service:SwitchPower:1");
        service.setServiceID("urn:schemas-upnp-org:serviceId:SwitchPower:1");

        HostedStateVariable stateVariableTarget =
            service.createStateVariable("A_ARG_TYPE_Target", "boolean", "0", "0", false);
        // fix for Whistler (can't understand A_ARG_TYPE)
        //   service.createStateVariable("Target", "boolean", "0", "0", false);
        service.addStateVariable(stateVariableTarget);

        HostedStateVariable stateVariableStatus =
            service.createStateVariable("Status", "boolean", "0", "0", true);
        service.addStateVariable(stateVariableStatus);

        Action actionSetTarget = service.createAction("SetTarget");
        Argument argument = new Argument(
            "newTargetValue", Argument.IN, false, stateVariableTarget.getName());
        actionSetTarget.addArgument(argument);
        service.addAction(actionSetTarget);

        Action actionGetStatus = service.createAction("GetStatus");
        argument = new Argument(
            "ResultStatus", Argument.OUT, true, stateVariableStatus.getName());
        actionGetStatus.addArgument(argument);
        service.addAction(actionGetStatus);

        this.addService(service);
    }


    public static void main(String[] args) {
        String uuid = null;
        String friendlyName = null;

        if (args.length != 0)
            uuid = args[0];
        if (args.length > 1)
            friendlyName = args[1];

        // set the appropriate UPnP Constants
        String webDir = System.getProperty("user.dir") + "\\www\\";

        UPnPConstants.WEB_ROOT_DIR = webDir;

        // optional, standard is already 80
        UPnPConstants.WEB_PORT = 80;

        // if true, Microsoft Millenium will recognize device
        // but SSDP messages will not be 100% UPnP compliant
        UPnPConstants.MS_MILLENIUM_COMPATIBLE = true;

        // SSDP lease time in seconds, default 1800
        UPnPConstants.SSDP_LEASE_TIME = 300;

        // SSDP time to live
        UPnPConstants.SSDP_TTL = 1;

        // set the Debug level to normal output
        Debug.setDebugLevel(Debug.NORMAL);

        BinaryLightDevice device;

        // create a device manually
        device = new BinaryLightDevice(uuid, friendlyName);

        // or create the device from an XML File
        /*
        try {
            File descriptionFile = new File(webDir + "../TestDevice.xml");
            device = BinaryLightDevuce(descriptionFile);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
        */
    }

    // Example how to write the XML device description to a file
    private void writeXML(Device device) {

        try {
            File file = new File("DeviceDescription.xml");
            FileWriter writer = new FileWriter(file);
            writer.write(device.getDeviceDescription());
            writer.close();
            Debug.println(MessageLevel.STANDARD,
                "File written: " + file.getAbsolutePath());
        }
        catch (java.io.IOException e) {
            e.printStackTrace();
        }
    }
}