/**
 * Title:        SwitchPower Implementation<p>
 * Description:  Implementation for switch power service<p>
 * Copyright (c) 2001, Siemens AG<p>
 *
 * @author Markus Alexander Wischy
 * @version 1.0
 */


package com.siemens.upnp.testdevice.service;

import java.util.Vector;

import com.siemens.upnp.Action;
import com.siemens.upnp.Argument;
import com.siemens.upnp.ArgumentList;
import com.siemens.upnp.SOAPException;
import com.siemens.upnp.devicehost.HostedDevice;
import com.siemens.upnp.devicehost.HostedService;
import com.siemens.upnp.devicehost.HostedStateVariable;
import com.siemens.upnp.devicehost.UPnPActionListener;
import com.siemens.upnp.soap.SOAPHelper;

// This stub implementation is rather tedious, could (should) be generated by a
// SOAP stub generator that takes the SwitchPower Interface as input

public class SwitchPowerImpl implements SwitchPower, UPnPActionListener  {

    private Vector listeners = new Vector();
	
    private Action actionSetTarget;
    private Action actionGetStatus;
    private Action actionSetStatus;
    private HostedStateVariable stateVariableTarget;
    private HostedStateVariable stateVariableStatus;

    public SwitchPowerImpl(HostedDevice device) {

        // get the references of the actions and variables of the service
        HostedService powerService =
            (HostedService)device.getServiceByID("SwitchPower:1");

        // no correct service found, should throw an exception
        if (powerService == null)
            return;

        // add UPnP action listener to the power switch service
        powerService.setActionListener(this);

        // initialize member variables
        actionSetTarget = powerService.getActionByName("SetTarget");
        actionGetStatus = powerService.getActionByName("GetStatus");
        actionSetStatus = powerService.getActionByName("SetStatus");

        stateVariableTarget =
            (HostedStateVariable)powerService.getVariableByName("Target");
        stateVariableStatus =
            (HostedStateVariable)powerService.getVariableByName("Status");
        stateVariableTarget.setValue("0");
        stateVariableStatus.setValue("0");
    }

    // Implementation of SwitchPower Interface
    public void setTarget(boolean newTarget) {
        // update the state variables
        stateVariableTarget.setValue(SOAPHelper.convertToString(newTarget));
        stateVariableStatus.setValue(SOAPHelper.convertToString(newTarget));
        this.fireEvents(newTarget);
    }

    // Implementation of SwitchPower Interface
    public boolean getStatus() {
        try {
            return SOAPHelper.convertToBoolean(stateVariableStatus.getValue());
        }
        catch (SOAPException e) {
            return false;
        }
    }

    // implementation of UPnPActionListener Interface
    public void processActionRequest(Action action) throws SOAPException {

        // get the input and output arguments
        ArgumentList inArgumentList = action.getArguments().getInputArguments();
        ArgumentList outArgumentList = action.getArguments().getOutputArguments();

        // print this actions request (for debug purposes)
        System.out.print("Action request: " + action.getName() + "(");
        for (int i = 0; i < inArgumentList.size(); i++) {
            Argument argument = inArgumentList.elementAt(i);
            if (i != 0) {
                System.out.print(", ");
            }
            System.out.print(argument.getRelatedStateVariable() + " " + argument.getValue());
        }
        System.out.println(")");

        // GetStatus
        if (action.getName().equalsIgnoreCase("GetStatus")) {
            if (outArgumentList.size() == 1) {
                Argument newStatus = outArgumentList.elementAt(0);
                newStatus.setValue(stateVariableStatus.getValue());
                return;
            }
            else
                throw new SOAPException(SOAPHelper.INVALID_ARGS, "Invalid Arguments: too many or too few");
        }

        // SetStatus
        else if (action.getName().equalsIgnoreCase("SetTarget")) {
            if (inArgumentList.size() == 1) {
                Argument newTarget = inArgumentList.elementAt(0);
                try {
                    boolean arg1 = SOAPHelper.convertToBoolean(newTarget.getValue());
                    this.setTarget(arg1);
                    return;
                }
                catch (SOAPException e) {
                    System.out.println(e.getMessage());
                    throw new SOAPException(SOAPHelper.INVALID_ARGS,
                        "Target Argument not boolean type: " + newTarget.getValue());
                }
            }
            else {
                throw new SOAPException(SOAPHelper.INVALID_ARGS,
                    "Invalid Arguments: too many or too few");
            }
        }
        throw new SOAPException(SOAPHelper.INVALID_ACTION,
            "Invalid Action: " + action.getName());
    }

    // for switch power listener support
    public void addListener(SwitchPower listener) {
        listeners.addElement(listener);
    }

    public void removeListener(SwitchPower listener) {
        listeners.remove(listeners);
    }

    private void fireEvents(boolean newTarget) {
        for (int i=0; i < listeners.size(); i++) {
            SwitchPower listener =
                (SwitchPower)listeners.elementAt(i);
            listener.setTarget(newTarget);
        }
    }
}
