/**
 * Title:       Binary Light Frame<p>
 * Description: User Interface for Binary Light Device<p>
 * Copyright (c) 2001, Siemens AG
 *
 * @author Markus Alexander Wischy
 * @version 1.0
 */

package com.siemens.upnp.testdevice.ui;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;

import com.siemens.upnp.InvalidDescriptionException;
import com.siemens.upnp.testdevice.BinaryLightDevice;
import com.siemens.upnp.util.Tracer;
import com.siemens.upnp.util.Tracers;

public class BinaryLightFrame extends Frame
        implements ActionListener, ChangeListener {

    protected String imageDir;
    protected final String LABEL_ANNOUNCE = "Announce device";
    protected final String LABEL_REMOVE = "Remove device";

    protected Image lampOnImage = null;
    protected Image lampOffImage = null;
    protected Image frameIcon = null;

    protected ImageButton lampButton = null;
    protected Button announceButton = new Button(LABEL_ANNOUNCE);
    protected Button onOffButton = null;

    protected BinaryLightDevice upnpDevice;
    
    protected static Tracer tracer = Tracer.getTracer("com.siemens.upnp.testdevice"); 

    public BinaryLightFrame(BinaryLightDevice device) {

        upnpDevice = device;

        this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        this.initUI();

        this.pack();
        this.setVisible(true);
    }

    protected void initUI() {
        this.setSize(new Dimension(240, 320));
        this.setTitle("Siemens WinCE 3.0 TestDevice");
        this.setLayout(new BorderLayout());

        Panel picturePanel = new Panel();

        // load images
        this.loadImages();

        this.setIconImage(frameIcon);
        lampButton = new ImageButton(lampOnImage, lampOffImage, false);
        lampButton.addChangeListener(this);
        picturePanel.add(lampButton);

        Panel buttonPanel = new Panel(new FlowLayout());

        onOffButton = new Button("On");
        onOffButton.setSize(30,12);
        onOffButton.addActionListener(this);
        buttonPanel.add(onOffButton);

        announceButton.addActionListener(this);
        buttonPanel.add(announceButton);

        this.add(picturePanel, BorderLayout.CENTER);
        this.add(buttonPanel, BorderLayout.SOUTH);
    }

    protected void loadImages() {
        String userDir = System.getProperty("user.dir");

        // JDK with current dir in \classes
        String imageSubDir1 = "\\..\\www\\";
        // JBuilder with current dir in \sources\package\
        String imageSubDir2 = "\\..\\..\\www\\";
        // JAR file???
        String imageSubDir3 = "\\www\\";
        String imageSubDir = imageSubDir3;

        File file1 = new File(userDir + imageSubDir1 + "icon.gif");
        File file2 = new File(userDir + imageSubDir2 + "icon.gif");
        File file3 = new File(userDir + imageSubDir3 + "icon.gif");
        if (file1.exists())
            imageSubDir = imageSubDir1;
        else if (file2.exists())
            imageSubDir = imageSubDir2;
        else if (file3.exists())
            imageSubDir = imageSubDir3;
        else {
			if (Tracers.debug) tracer.info("Images not found! Was looking in:");
			if (Tracers.debug) tracer.info( "  " + file1.getPath());
			if (Tracers.debug) tracer.info( "  " + file2.getPath());
			if (Tracers.debug) tracer.info( "  " + file3.getPath());
        }

        imageDir = userDir + imageSubDir;
        frameIcon = Toolkit.getDefaultToolkit().getImage(userDir + imageSubDir + "icon.gif");
        lampOnImage = Toolkit.getDefaultToolkit().getImage(userDir + imageSubDir + "lampOn.gif");
        lampOffImage = Toolkit.getDefaultToolkit().getImage(userDir + imageSubDir + "lampOff.gif");

        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(lampOffImage, 0);
        tracker.addImage(lampOnImage, 1);
        tracker.addImage(frameIcon, 2);
        try {
            tracker.waitForAll();
        }
        catch (InterruptedException e) {};

    }

    public String getImageDir() {
        return imageDir;
    }

    // event from UPnP binary light, update user interface
    public void updateUI(boolean newTarget) {
        if (newTarget == true) {
            onOffButton.setLabel("Off");
        }
        else {
            onOffButton.setLabel("On");
        }
        lampButton.setStatus(newTarget);
    }

    // user event from Image Button
    public void stateChanged(Component source, boolean newStatus) {
		if (Tracers.debug)
			tracer.info( "State changed to: " + newStatus);
        if (newStatus == true) {
            onOffButton.setLabel("Off");
        }
        else {
            onOffButton.setLabel("On");
        }
        upnpDevice.updateStatus(newStatus);
    }

    // user event from standard button
    public void actionPerformed(ActionEvent event) {

        // image button pressed
        if (event.getSource() == onOffButton) {
            String currentLabel = onOffButton.getLabel();
            if (currentLabel.equals("On")) {
                onOffButton.setLabel("Off");
                lampButton.setStatus(true);
                upnpDevice.updateStatus(true);
            }
            else {
                onOffButton.setLabel("On");
                lampButton.setStatus(false);
                upnpDevice.updateStatus(false);
            }
        }

        // announce/remove button pressed
        else if (event.getSource() == announceButton) {
            String currentLabel = announceButton.getLabel();
            if (currentLabel.equals(LABEL_ANNOUNCE)) {
                try {
                    // announce the device in the UPnP network
                    upnpDevice.announce();
                    announceButton.setEnabled(false);
                    try {
                        // sleep, as long as messages are sent
                        Thread.sleep(
                            com.siemens.upnp.ssdp.SSDPConstants.RESEND_DELAY
                            * (com.siemens.upnp.ssdp.SSDPConstants.MESSAGE_RESEND - 1)
                            * 1000);
                    }
                    catch (InterruptedException e) {}
                    announceButton.setLabel(LABEL_REMOVE);
                    announceButton.setEnabled(true);
                }
                catch (InvalidDescriptionException e) {
					tracer.error( "Invalid Device Description: " + e.getMessage());
                }
            }
            else {
                upnpDevice.remove();
                announceButton.setEnabled(false);
                try {
                    // sleep, as long as messages are sent
                    Thread.sleep(
                        com.siemens.upnp.ssdp.SSDPConstants.RESEND_DELAY
                        * (com.siemens.upnp.ssdp.SSDPConstants.MESSAGE_RESEND - 1)
                        * 1000);
                }
                catch (InterruptedException e) {}
                announceButton.setLabel(LABEL_ANNOUNCE);
                announceButton.setEnabled(true);
            }
        }
    }

    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            // remove the device from the UPnP network
            upnpDevice.remove();
            upnpDevice.shutDown();
            System.exit(0);
        }
    }
}